require_dependency "rechn_mgm_user_area/application_controller"

module RechnMgmUserArea
    class OrganisationsController < AuthenticatedController
        layout 'application'

        before_action :set_user, only: [:resend_invitation]
        before_action :set_membership, only: [:index, :create_invitation]

        def index
            @my_organisation = current_user.current_organisation
            @member_ships = nil
            if current_user.current_membership.is_primary?
                @member_ships = current_user.member_ships
            end
            @users = current_user.users_of_current_organisation
            @invited_users = current_user.invited_by_organisation
        end


        def new_invitation
            respond_to do |format|
                format.html
                format.js
            end
        end

        def create_invitation
            invited_user = DeviseAuthentication::User.where(email: organisation_params[:email]).first

            if invited_user.present?
                if invited_user.id == current_user.id
                    flash[:error] = I18n.t "You cannot add yourself!"
                    redirect_to user_root_path
                    return
                end

                if current_user.user_ids_of_current_organisation.include? invited_user.id
                    flash[:error] = I18n.t "This user is already in your organisation!"
                    redirect_to user_root_path
                    return
                end

                if current_user.invited_by_me.map{|u| u.id}.include? invited_user.id
                    flash[:error] = I18n.t "You already invited this user!"
                    redirect_to user_root_path
                    return
                end

                # User is already known: Add him directly
                success = current_user.add_user_to_current_organisation(invited_user.id)
                if success
                    flash[:info] = I18n.t "The invited user was added"
                else
                    flash[:error] = I18n.t "The invited user could not be added. Please contact administrator!"
                end
            else
                # User is not known: Invite him
                flash[:info] = I18n.t("invitation_sent", email: organisation_params[:email])
                DeviseAuthentication::User.invite!({email: organisation_params[:email], forename: organisation_params[:first_name], lastname: organisation_params[:last_name], invited_for_organisation_id: current_user.current_organisation.id}, current_user)
            end
            redirect_to user_root_path
        end

        def resend_invitation
            flash[:info] = I18n.t("invitation_sent", email: @user.email)
            DeviseAuthentication::User.invite!({email: @user.email, forename: @user.forename, lastname: @user.lastname}, current_user)

            redirect_to user_root_path
        end

        def new_organisation
            respond_to do |format|
                format.html
                format.js
            end
        end

        def create_organisation
            new_organisation = RechnMgmUserArea::Organisation.new(name: new_organisation_params[:name])
            if new_organisation.save
                new_membership = RechnMgmUserArea::MemberShip.new(user_id: current_user.id, organisation_id: new_organisation.id, roles_mask: 1, is_primary: true)
                if new_membership.save
                    flash[:info] = I18n.t("new_organisation_created")
                    session[:selected_organisation] = new_membership.id
                else
                    flash[:error] = I18n.t("new_organisation_not_created")
                    new_organisation.destroy
                end
            end
            redirect_to user_root_path
        end

        def rename_organisation_screen
            # TODO: Refactor.. @current_membership_id should be set by before_action!
            @current_organisation = current_user.current_organisation
            @current_membership_id = session[:selected_organisation] || current_user.current_membership_id
            @my_organisation = RechnMgmUserArea::MemberShip.find( @current_membership_id ).organisation
            @current_name = @my_organisation.name

            respond_to do |format|
                format.html
                format.js
            end
        end

        def rename_organisation
            membership = RechnMgmUserArea::MemberShip.find( rename_organisation_params[:membership_id] )
            organisation = membership.organisation
            organisation.name =  rename_organisation_params[:name]

            if organisation.save
                flash[:info] = I18n.t("new_organisation_renamed")
            else
                flash[:error] = I18n.t("new_organisation_not_renamed")
            end

            redirect_to user_root_path
        end

        def switch_organisation
            session[:selected_organisation] = switch_organisation_params[:selected_organisation]
            # TODO: Shouldn't I set the current organisation here as well? current_user.current_organisation = .. ???
            render :json => { :error => "None", "ErrorCode" => 0 }
        end

        private
        def organisation_params
            params.require(:invitation).permit(:email, :first_name, :last_name)
        end

        def new_organisation_params
            params.require(:organisation).permit(:name)
        end

        def rename_organisation_params
            params.require(:organisation).permit(:name, :membership_id)
        end

        def switch_organisation_params
            params.permit(:selected_organisation)
        end

        def set_user
            @user = DeviseAuthentication::User.find(params[:user_id])
        end

        def set_membership
            current_user.current_membership_id = session[:selected_organisation] if session[:selected_organisation].present?
        end
    end
end
